/*
 * This macro allows you to use a 'C' function to handle a 68HC11 interrupt.
 *
 * The only parameter is the DECIMAL address of the interrupt vector.
 * NOTE: If you are debugging with a monitor or other software which re-
 * vectors the interrupts through a RAM vector table, be sure to use the
 * address of the RAM vector, NOT the actual CPU vector which will likely
 * be in ROM.
 *
 * The macro MUST be placed IMMEDIATELY preceeding the definition of the
 * function which is to be the interrupt handler. Example:
 *
 * 		INTERRUPT(_SCI_) serial_int_handler()
 * 		{
 *			/* ... Function code to handle serial interrupt ... */
 *		}
 *
 *
 * *Note1:	If the '?parm' and '?temp' locations are NOT in the zero
 *			page of memory, add 4 to the BSR offset.
 *
 * *Note2:	Some monitors (such as 'buffalo') re-vector the interrupts
 *			using JMP instructions in the RAM vector table instead of
 *			vectors (like MON11 and the 68HC11 CPU itself do). If you
 *			are using such a debugger, you need to change the FDB near
 *			the beginning of the macro to JMP.
 *
 * This file **REQUIRES** the extended pre-processor (MCP).
 * (CC11 ... -P, or "Preprocess" step enabled in DDSIDE)
 *
 * Copyright 1991-2005 Dave Dunfield
 * All rights reserved.
 */
#define	_SP_				/* Null separator */
#define	INTERRUPT(vec) asm {\
$SE:0						/* Insure output in code segment */\
_/**/vec	/* This label will be the address of the "prefix" stub */\
		ORG		vec			/* Position to interrupt vector */\
		FDB		_/**/vec	/* Enter the label address */\
		ORG		_/**/vec	/* Position back to original PC */\
		LDX		_SP_?parm	/* Get runtime parameter */\
		PSHX				/* Save during interrupt */\
		LDX		_SP_?temp	/* Get temporary location */\
		PSHX				/* Save during interrupt */\
		BSR		_/**/vec+15	/* Invoke 'C' interrupt handler*/\
		PULX				/* Restore temporary location value*/\
		STX		_SP_?temp	/* Replace value */\
		PULX				/* Restore runtime parameter*/\
		STX		_SP_?parm	/* Replace value */\
		RTI					/* End interrupt */\
}
#undef _SP_

/*
 * Common 68HC11 interrupt vector addresses (Must be in DECIMAL)
 */
#define	_SCI_	65494		/* Serial Communications Interface */
#define	_SPI_	65496		/* Serial Peripheral Interface */
#define	_PACCI_	65498		/* Pulse Accumulator Input */
#define	_PACCO_	65500		/* Pulse Accumulator Overflow */
#define	_TOVF_	65502		/* Timer Overflow */
#define	_TI4O5_	65504		/* Timer Input Capture 4 / Output Compare 5 */
#define	_TOC4_	65506		/* Timer Output Compare 4 */
#define	_TOC3_	65508		/* Timer Output Compare 3 */
#define	_TOC2_	65510		/* Timer Output Compare 2 */
#define	_TOC1_	65512		/* Timer Output Compare 1 */
#define	_TIC3_	65514		/* Timer Input Capture 3 */
#define	_TIC2_	65516		/* Timer Input Capture 2 */
#define _TIC1_	65518		/* Timer Input Capture 1 */
#define	_RT_	65520		/* Real Time Interrupt */
#define	_IRQ_	65522		/* General Interrupt Request */
#define	_XIRQ_	65524		/* Non-Maskable interrupt */
#define	_SWI_	65526		/* Software Interrupt */
#define	_ILLOP_	65528		/* Illegal Opcode */
#define	_COP_	65530		/* COP Watchdog */
#define	_CLOCK_	65532		/* Clock Monitor */

